@@ -57,6 +57,7 @@ start_animation (void)
 
 	druid_data.throbber_id = gtk_timeout_add (150, animate, NULL);
 	gtk_widget_show (GET_WIDGET ("gdb-progress"));
+	gtk_widget_set_sensitive (GET_WIDGET ("gdb-copy-save-box"), FALSE);
 }
 
 static void
@@ -67,6 +68,7 @@ stop_animation (void)
 	gtk_timeout_remove (druid_data.throbber_id);
 	druid_data.throbber_id = 0;
 	gtk_widget_hide (GET_WIDGET ("gdb-progress"));
+	gtk_widget_set_sensitive (GET_WIDGET ("gdb-copy-save-box"), TRUE);
 }
 
 void
@@ -125,10 +127,20 @@ stop_gdb (void)
 	}
 	
 	g_io_channel_shutdown (druid_data.ioc, 1, NULL);
+	
+	kill (druid_data.gdb_pid, SIGTERM);
+	/* i don't think we need to SIGKILL it */
+	/*kill (druid_data.gdb_pid, SIGKILL);*/
 	waitpid (druid_data.gdb_pid, NULL, 0);
 	
 	druid_data.gdb_pid = 0;
 
+	/* sometimes gdb doesn't restart the old app... */
+	if (druid_data.app_pid) {
+		kill (druid_data.app_pid, SIGCONT);
+		druid_data.app_pid = 0;
+	}
+
 	druid_set_sensitive (FALSE, TRUE, TRUE);
 	stop_animation ();
 
@@ -213,7 +225,7 @@ handle_gdb_input (GIOChannel *ioc, GIOCondition condition, gpointer data)
 {	
 	gboolean retval = FALSE;
 	gchar buf[1024];
-	guint len;
+	gsize len;
 	GIOStatus io_status;
 
  gdb_try_read:
@@ -236,12 +248,16 @@ handle_gdb_input (GIOChannel *ioc, GIOCondition condition, gpointer data)
 		GtkTextIter end;
 		GtkTextBuffer *buffy;
 		GtkTextView *tv;
+		char *utftext;
+		gsize utflen;
 
 		tv = GTK_TEXT_VIEW (GET_WIDGET ("gdb-text"));
 		buffy = gtk_text_view_get_buffer (tv);
 
 		gtk_text_buffer_get_end_iter (buffy, &end);
-		gtk_text_buffer_insert (buffy, &end, buf, len);
+		utftext = g_locale_to_utf8 (buf, len, NULL, &utflen, NULL);
+		gtk_text_buffer_insert (buffy, &end, utftext, utflen);
+		g_free (utftext);
 	}
 
 	if (!retval)
@@ -255,7 +271,9 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 {
 	GtkWidget *d;
 	char *s;
-	char *app2;
+	const char *short_app;
+	char *long_app;
+	GError *error = NULL;
 	char *args[] = { "gdb",
 			 "--batch", 
 			 "--quiet",
@@ -282,21 +300,30 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 	if (!app || !extra || !*app || !*extra)
 		return;
 	
-	/* FIXME: we should probably be fully expanding the link to
-	   see if it is a directory, but unix sucks and i am lazy */
-	if (g_file_test (app, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
-		app2 = g_strdup (app);
-	else
-		app2 = g_find_program_in_path (app);
 
-	if (!app2) {
+	if (*app == G_DIR_SEPARATOR) {
+		long_app = g_strdup (app);
+		short_app = strrchr (app, G_DIR_SEPARATOR) + 1;
+	} else {
+		long_app = g_find_program_in_path (app);
+		if (!long_app) {
+			/* Applets are not in path... */
+			long_app = g_strconcat(g_getenv("GNOME_PATH"), "/libexec/", app);
+		}
+		short_app = app;
+	}	
+
+	g_free (druid_data.current_appname);
+	druid_data.current_appname = g_strdup (short_app);
+
+	if (!long_app) {
 		g_free (args[0]);
 		return;
 	}
 
-	args[4] = app2;
+	args[4] = long_app;
 
-	d(g_message ("About to debug '%s'", app2));
+	d(g_message ("About to debug '%s'", long_app));
 	
 	if (!g_file_test (BUDDY_DATADIR "/gdb-cmd", G_FILE_TEST_EXISTS)) {
 		d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
@@ -309,32 +336,33 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 		gtk_dialog_set_default_response (GTK_DIALOG (d),
 						 GTK_RESPONSE_OK);
 		gtk_widget_destroy (d);
-		g_free (app2);
+		g_free (long_app);
 		return;
 	}
 
-	/* FIXME: use GError */
 	if (!g_spawn_async_with_pipes (NULL, args, NULL, 0, NULL, NULL,
 				       &druid_data.gdb_pid,
 				       NULL, 
 				       &druid_data.fd, 
-				       NULL, NULL)) {
+				       NULL, &error)) {
 		d = gtk_message_dialog_new (GTK_WINDOW (GTK_WIDGET ("druid-window")),
 					    0,
 					    GTK_MESSAGE_ERROR,
 					    GTK_BUTTONS_OK,
-					    _("There was an error running gdb."));
+					    _("There was an error running gdb:\n\n%s"),
+					    error->message);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_dialog_set_default_response (GTK_DIALOG (d),
 						 GTK_RESPONSE_OK);
 		gtk_widget_destroy (d);
-		g_free (app2);
+		g_error_free (error);
+		g_free (long_app);
 		return;
 	}
 	
 	druid_data.ioc = g_io_channel_unix_new (druid_data.fd);
 	
-	s = g_strdup_printf ("Core was generated from '%s'\n\n", app2);
+	s = g_strdup_printf ("Backtrace was generated from '%s'\n\n", long_app);
 	buddy_set_text ("gdb-text", s);
 	g_free (s);
 	g_io_add_watch (druid_data.ioc, G_IO_IN | G_IO_HUP,
@@ -349,5 +377,5 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 
 	druid_data.explicit_dirty = FALSE;
 
-	g_free (app2);
+	g_free (long_app);
 }
